'''
# Copyright (C) Labomedia April 2012
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#############################################################################
'''

import GameLogic as gl
import socket
from OSC3.OSC import decodeOSC

# init global var
ip = gl.ip
port = gl.port_in
buffer_size = 1024

stack =[] # player order for display in blender, a little bit complexed, but not complicated
data = [] # osc data with modif along the script
players_data = [] # list of list
master = 0  # item in players_data who master = 1, gl.master=true if I'm master
            # master and gl.master are not the same variable !!
            # stupid, yes, I agree
# level is the players number and the scene
level = 2

# Get controller and owner
controller = gl.getCurrentController()
owner = controller.owner
# Get all scenes
scenes = gl.getSceneList()
# For all scenes, all objets, all prop
# Get bat and ball
for scen in scenes:
    for obj in scen.objects:
        if str(obj) == "2_bat_1" :
            bat_1 = obj
        if str(obj) == "ball2":
            ball2 = obj

def main():
    data = get_osc()
    i_quit(data)
    #data=['/for_all_players',',siiiffffsiiiffff','10.0.0.5',0,8,0,7,1,5,-1 ,
    #                                             '10.0.0.4',1,9,10,9,0,0,0 ]

    players_data = set_players_data_with_data(data)
    # players_data = [ ['10.0.0.5',0,8,0,7,1,5,1], 8 items
    #                  master ['10.0.0.4',1,9,10,9,0,8,0] ]
    #            ip,master,number,score,batx,baty,ballx,bally

    # stack is complicated
    stack, master = create_players_stack(players_data)

    print_usefull_var((players_data, stack, master))
    # [['10.0.0.5', 0, 8, 0, 7, 1, 5, -1], ['10.0.0.4', 1, 9, 10, -9, -0, -2, -0]], [9, 8], 0)

    # Set ball
    if not gl.master:
        # disable ball rigid body with prop master=False in ball
        ball2["master"] = False
        if master > 0:
            set_ball_position(players_data, stack, master)

    # Always set bat of other players
    # ok only with 2 players
    set_bat_position(players_data, stack, master)

def set_bat_position(players_data, stack, master):
    if len(stack) == 2:
        other = stack[1]
        for p_list in players_data:
            if p_list[2] == other:
                bat_1.localPosition = [-p_list[4], -p_list[5], 1]

def set_ball_position(players_data, stack, master):
    # who is the master: master position of the list in the global list
    ball2.localPosition = [-players_data[master][6], -players_data[master][7], 1]

def get_osc():
    global data
    # Connect Blender
    if not gl.connected :
        gl.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        try:
            gl.sock.bind((ip, port))
            gl.sock.setblocking(0)
            gl.sock.settimeout(0.005)
            gl.sock.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, buffer_size)
            print('Get OSC on IP={} Port={} Buffer={}'.format(ip, port, buffer_size))
            gl.connected = True
        except:
            print('No connected to IP={} Port={}'.format(ip, port))
            pass

    # If Blender connected
    if gl.connected :
        try:
            raw_data = gl.sock.recv(buffer_size)
            #print(raw_data)
            data = decodeOSC(raw_data)
            #print(data)
        except socket.error:
            pass
    return data

def chunks(l, n):
    # coupe en n listes de liste de len l
    return [l[i:i+n] for i in range(0, len(l), n)]

def set_players_data_with_data(data):
    # a list of data of all players
    global players_data
    if '/for_all_players' in data :
        data = data[2:]
        if len(data) % 8 == 0 : # verif pour qu il ne manque pas de data
            players_data = chunks(data, 8) # division en sous liste
    global level
    level = len(players_data)
    return players_data

def create_players_stack(players_data):
    player_list = []
    global master
    global stack
    for i in range(len(players_data)):
        player_list.append(players_data[i][2])
        if players_data[i][1] == 1:
            master = i #  master position of the list in the global list
        if players_data[i][0] == gl.local_ip: # where is my ip
            gl.player = players_data[i][2] # this is my player number
            # I'm master or slave ?
            if players_data[i][1] == 1:
                gl.master = True # My physics motor is used for all players
            else :
                gl.master = False # I am not master, set ball position
    # I sort in ascending order [5, 1, 9, 6]
    if len(player_list) > 1:
        player_list.sort()
        # where is my player number in list
        ix = player_list.index(gl.player)
        # I cut the list after my number
        a = player_list[ix:]
        # I cut the list before my number
        b = player_list[:ix]
        # The stack
        stack = a + b
    return stack, master

def print_usefull_var(usefull_var):
    # print every 60 frames
    gl.tp += 1
    if gl.tp == 60:
        print(usefull_var)
        gl.tp = 0

def i_quit(data):
    # very bad atchtung
    global players_data
    if '/quit' in data :
        owner["quit"] = True
        gl.sock.close()

main()
